package com.lj.web.config;

import org.elasticsearch.client.transport.TransportClient;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.common.transport.TransportAddress;
import org.elasticsearch.transport.client.PreBuiltTransportClient;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.LinkedList;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class EsPool {

    private static LinkedList<TransportClient> clientPool = new LinkedList<>();
    private static final int INIT_CONNECTIONS = 2;
    private Lock lock = new ReentrantLock();
    private Condition condition = lock.newCondition();

    static {
        for (int i = 0; i < INIT_CONNECTIONS; i++) {
            try {
                Settings settings = Settings.builder()
                        .put("client.transport.sniff", true)
                        .put("cluster.name", "es")
                        .build();
                System.setProperty("es.set.netty.runtime.available.processors", "false");
                TransportClient client = new PreBuiltTransportClient(settings)
                        .addTransportAddress(new TransportAddress(InetAddress.getByName("192.168.50.32"), 9300))
                        .addTransportAddress(new TransportAddress(InetAddress.getByName("192.168.50.39"), 9300))
                        .addTransportAddress(new TransportAddress(InetAddress.getByName("192.168.50.34"), 9300));

                clientPool.addLast(client);
            } catch (UnknownHostException e) {
                e.printStackTrace();
            }
        }
    }


    // 获取数据库连接
    public TransportClient getClient() {
        TransportClient conn = null;
        lock.lock();
        try {
            while (clientPool.size() < 0) {
                try {
                    condition.await();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
            if (!clientPool.isEmpty()) {
                conn = clientPool.removeFirst();
            }
            return conn;
        } finally {
            lock.unlock();
        }
    }

    // 释放数据库连接
    public void releaseConnection(TransportClient conn) {
        if (conn != null) {
            lock.lock();
            try {
                // 释放连接过程就是把连接放回连接池过程
                clientPool.addLast(conn);
                condition.signal();
            } finally {
                lock.unlock();
            }
        }
    }

}
