package com.lj.web.server;


import com.lj.web.config.ClickHouseDruid;
import com.lj.web.config.EsPool;
import com.lj.web.config.SqlServerDruid;
import com.lj.web.entity.AccessLog;
import com.lj.web.utils.DateUtils;
import com.lj.web.utils.SnowflakeIdWorker;
import org.elasticsearch.action.index.IndexRequestBuilder;
import org.elasticsearch.client.transport.TransportClient;
import org.elasticsearch.rest.RestStatus;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Date;

import static org.elasticsearch.common.xcontent.XContentFactory.jsonBuilder;


/**
 * @Author: 黄树文
 * @Date: 2019/12/14 13:29
 */
public class LogServlet extends javax.servlet.http.HttpServlet {


    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {

        AccessLog accessLog = getAccessLog(request);

        System.out.println(accessLog);

        try {
//            insertIntoSqlserver(accessLog);
//            insertIntoEs(accessLog);
            insertIntoClickhouse(accessLog);

        } catch (SQLException e) {
            e.printStackTrace();
        }
        System.out.println("插入成功");

    }


    @Override
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws IOException {
        doGet(request, response);
    }


    // 将log封装成对象
    private AccessLog getAccessLog(HttpServletRequest request) throws UnsupportedEncodingException {
        String qs = request.getQueryString();

        System.out.println(qs);
        qs = URLDecoder.decode(qs, "utf-8");
        System.out.println(qs);
        String[] kvs = qs.split("&");

        AccessLog accessLog = new AccessLog();

        accessLog.setId(new SnowflakeIdWorker(0, 0).nextId().toString());
        accessLog.setUrl(kvs[0].split("=").length < 2 ? "" : kvs[0].split("=")[1]);
        accessLog.setUrlname(kvs[1].split("=").length < 2 ? "" : kvs[1].split("=")[1]);
        accessLog.setTitle(kvs[2].split("=").length < 2 ? "" : kvs[2].split("=")[1]);
        accessLog.setCharset(kvs[3].split("=").length < 2 ? "" : kvs[3].split("=")[1]);
        accessLog.setScreen(kvs[4].split("=").length < 2 ? "" : kvs[4].split("=")[1]);
        accessLog.setColor(kvs[5].split("=").length < 2 ? "" : kvs[5].split("=")[1]);
        accessLog.setLanguage(kvs[6].split("=").length < 2 ? "" : kvs[6].split("=")[1]);
        accessLog.setJvm_enabled(kvs[7].split("=").length < 2 || !"0".equals(kvs[7].split("=")[1]));
        accessLog.setCookie_enabled(kvs[8].split("=").length < 2 || !"0".equals(kvs[8].split("=")[1]));
        accessLog.setFlash_ver(kvs[9].split("=").length < 2 ? "" : kvs[9].split("=")[1]);
        accessLog.setRef(kvs[10].split("=").length < 2 ? "" : kvs[10].split("=")[1]);
        accessLog.setUseragent(kvs[11].split("=").length < 2 ? "" : kvs[11].split("=")[1]);
        accessLog.setUv_id(kvs[12].split("=").length < 2 ? "0" : kvs[12].split("=")[1]);
        accessLog.setSs_id(kvs[13].split("=").length < 2 ? "0" : kvs[13].split("=")[1]);
        accessLog.setSs_count(kvs[14].split("=").length < 2 ? 0 : Integer.parseInt(kvs[14].split("=")[1]));
        accessLog.setSs_time(kvs[15].split("=").length < 2 ? new Date() : new Date(Long.parseLong(kvs[15].split("=")[1])));
        accessLog.setOpenid(kvs[16].split("=").length < 2 ? "" : kvs[16].split("=")[1]);
        accessLog.setUserPhone(kvs[17].split("=").length < 2 ? "" : kvs[17].split("=")[1]);
        accessLog.setIp(request.getRemoteAddr());

        return accessLog;

    }


    // 将log插入sqlserver
    private boolean insertIntoSqlserver(AccessLog accessLog) throws SQLException {
        System.out.println("开始插入数据");
        Connection connection = SqlServerDruid.getConn();
        String sql = "insert into mem_user_access_log(id,url,urlname,title,charset,color,screen,language,jvm_enabled,cookie_enabled," +
                "flash_ver,ref,useragent,uv_id,ss_id,ss_count,ss_time,openid,user_phone,ip) values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

        PreparedStatement ps = connection.prepareStatement(sql);
        ps.setString(1, accessLog.getId());
        ps.setString(2, accessLog.getUrl());
        ps.setString(3, accessLog.getUrlname());
        ps.setString(4, accessLog.getTitle());
        ps.setString(5, accessLog.getCharset());
        ps.setString(6, accessLog.getColor());
        ps.setString(7, accessLog.getScreen());
        ps.setString(8, accessLog.getLanguage());
        ps.setBoolean(9, accessLog.getJvm_enabled());
        ps.setBoolean(10, accessLog.getCookie_enabled());
        ps.setString(11, accessLog.getFlash_ver());
        ps.setString(12, accessLog.getRef());
        ps.setString(13, accessLog.getUseragent());
        ps.setString(14, accessLog.getUv_id());
        ps.setString(15, accessLog.getSs_id());
        ps.setInt(16, accessLog.getSs_count());
        ps.setObject(17, accessLog.getSs_time());
        ps.setString(18, accessLog.getOpenid());
        ps.setString(19, accessLog.getUserPhone());
        ps.setString(20, accessLog.getIp());

        boolean execute = ps.execute();

        ps.close();
        connection.close();
        return execute;

    }


    // 将log插入sqlserver
    private boolean insertIntoClickhouse(AccessLog accessLog) throws SQLException {
        System.out.println("开始插入数据");
        Connection connection = ClickHouseDruid.getConn();
        String sql = "insert into mem_user_access_log(id,url,urlname,title,charset,color,screen,language,jvm_enabled,cookie_enabled," +
                "flash_ver,ref,useragent,uv_id,ss_id,ss_count,ss_time,openid,user_phone,ip) values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

        PreparedStatement ps = connection.prepareStatement(sql);
        ps.setString(1, accessLog.getId());
        ps.setString(2, accessLog.getUrl());
        ps.setString(3, accessLog.getUrlname());
        ps.setString(4, accessLog.getTitle());
        ps.setString(5, accessLog.getCharset());
        ps.setString(6, accessLog.getColor());
        ps.setString(7, accessLog.getScreen());
        ps.setString(8, accessLog.getLanguage());
        ps.setBoolean(9, accessLog.getJvm_enabled());
        ps.setBoolean(10, accessLog.getCookie_enabled());
        ps.setString(11, accessLog.getFlash_ver());
        ps.setString(12, accessLog.getRef());
        ps.setString(13, accessLog.getUseragent());
        ps.setString(14, accessLog.getUv_id());
        ps.setString(15, accessLog.getSs_id());
        ps.setInt(16, accessLog.getSs_count());
        ps.setString(17, DateUtils.getFormatedTime(accessLog.getSs_time()));
        ps.setString(18, accessLog.getOpenid());
        ps.setString(19, accessLog.getUserPhone());
        ps.setString(20, accessLog.getIp());

        boolean execute = ps.execute();

        ps.close();
        connection.close();
        return execute;

    }

    // 将log插入elasticsearch
    private void insertIntoEs(AccessLog accessLog) throws IOException {
        EsPool esPool = new EsPool();
        TransportClient client = esPool.getClient();
        System.out.println(client == null);
        IndexRequestBuilder indexRequestBuilder = client.prepareIndex("mem_user_access_log", "_doc", accessLog.getId())
                .setSource(jsonBuilder().startObject()
                        .field("id", accessLog.getId())
                        .field("url", accessLog.getUrl())
                        .field("urlname", accessLog.getUrlname())
                        .field("title", accessLog.getTitle())
                        .field("charset", accessLog.getCharset())
                        .field("color", accessLog.getColor())
                        .field("screen", accessLog.getScreen())
                        .field("language", accessLog.getLanguage())
                        .field("jvm_enabled", accessLog.getJvm_enabled())
                        .field("cookie_enabled", accessLog.getCookie_enabled())
                        .field("flash_ver", accessLog.getFlash_ver())
                        .field("ref", accessLog.getRef())
                        .field("useragent", accessLog.getUseragent())
                        .field("uv_id", accessLog.getUv_id())
                        .field("ss_id", accessLog.getSs_id())
                        .field("ss_count", accessLog.getSs_count())
                        .field("ss_time", accessLog.getSs_time())
                        .field("openid", accessLog.getOpenid())
                        .field("user_phone", accessLog.getUserPhone())
                        .field("ip", accessLog.getIp()).endObject()
                );
        RestStatus status = indexRequestBuilder.get().status();
        System.out.println(status.getStatus());
        esPool.releaseConnection(client);
    }

}
